
import java.awt.*;
import java.util.*;
import simphy.*;

class IsingLattice extends Frame implements Runnable {
    int s[];
    int L = 30, dL = 4;   // dL = size of each cell
    int N = L*L;
    double T = 2.2;
    int cornerx = 10, cornery = 10;
    double Dcum=0,Ecum=0,Mcum=0,E2cum=0,M2cum=0;
    int Demon = 4;
    Image offscreen;
    Graphics buffer;
    Graphics screen;
    int M,E;
    RND rnd;
    double w[];
    int accept;
    int Lx,Ly;
    int nData = 0;
    boolean running = false;
    boolean MC = true;
    int t = 0;
    Thread runner;
    ControlFrame cf;

public IsingLattice(ControlFrame cf) {
      this.cf = cf;
      setTitle("Ising Model");
      setSize(300,300);
      setLocation(220,120);
      setVisible(true);
	  offscreen = createImage(getSize().width, getSize().height);
	  buffer = offscreen.getGraphics();
	  screen = getGraphics();
      rnd = new RND((long)(1000000*Math.random()));  // seed random number generator
      hot();
      start();
     }
     
public void paint(Graphics g) {
		setBackground(Color.white);
		g.drawImage(offscreen,0,0,this);
	}
	
    public void start() {
     runner = new Thread(this);
     runner.start();
    }

    public void stop() {
     runner.stop();
    }

    public void run() {
     while(true) {
         try {Thread.sleep(5);}  // delay 5 msec between updates
         catch (InterruptedException e){};
         if(running) 
            if(MC)
               runMC();
            else
               runDemon();
      }
    }


public void newLattice() {
       this.Lx = Lx;
       this.Ly = Ly;
       if ( L <= 10)
            dL = 20;
       else if ( L <= 20)
            dL = 10;
       else if ( L <= 80)
            dL = 4;             // set size of cells on screen
       else if (L <= 110)
            dL = 3;
       else 
            dL = 2;
       buffer.clearRect (0, 0,getSize().width, getSize().height);
       buffer.drawRect(10,10,dL*L,dL*L);
       N = L*L;
       s = new int[N];
       reset();
       boltzmann();
}

public void cold() {
       newLattice();
       for(int n = 0; n < N; n++) 
          s[n] = 1;
       E = -2*N;
       M = N;
       drawAllSpins();
 }

public void hot() {
       newLattice();
       E = 0;
       M = 0;
       for(int n = 0; n < N; n++) 
         if(rnd.nextDouble() > 0.5)
             s[n] = 1;
         else
             s[n] = -1;
       for(int n = 0; n < N; n++) {
           M += s[n];
           E -= s[n]*nnField(n);
       }
       E /= 2;   // divide by 2 because of double counting
       drawAllSpins();
 }
              
public void drawAllSpins() {          // draw occupied sites
     for(int i = 0; i < N; i++)
             drawSpin(buffer,i);
     buffer.setColor(Color.black);
     buffer.drawRect(cornerx,cornery,dL*L,dL*L);
     repaint();
}

    
public void drawSpin(Graphics g, int n) {
     int j = n / L;
     int i = n % L;
     if (s[n] == 1) 
         g.setColor(Color.red);
     else
         g.setColor(Color.green);
     g.fillRect(cornerx+i*dL,cornery+j*dL,dL,dL); 
}
 
public void boltzmann() {
     w = new double[9];
     for(int dE_over_2 = -4; dE_over_2 <= 4; dE_over_2++) 
        w[dE_over_2 + 4] = Math.exp(-2*dE_over_2/T);
}

public void runMC() {
     int dE_over_2,n;
     for (int istep = 0;istep < N;istep++) {  
         n = (int) (N*rnd.nextDouble());        // choose spin at random
         dE_over_2 = s[n]*nnField(n);
         if(w[dE_over_2 + 4] > rnd.nextDouble()) {
               s[n] = -s[n];
                   drawSpin(screen,n);
                   drawSpin(buffer,n);
                   E += 2*dE_over_2;
                   M += 2*s[n];
                   accept++;    
           }
         }
      collectData();  
}

public void runDemon() {
     int dE,n;
     for (int istep = 0;istep < N;istep++) {  
         n = (int) (N*rnd.nextDouble());        // choose spin at random
         dE = 2*s[n]*nnField(n);
         if(Demon >= dE) {
               s[n] = -s[n];
                   drawSpin(screen,n);
                   drawSpin(buffer,n);
                   E += dE;
                   M += 2*s[n];
                   Demon -= dE;
                   accept++;    
           }
         }
      collectData();  
}

public int nnField(int n) {  //sum spin neighbors using pbc
   int sum = 0;
   if (n < L) 
      sum += s[n + N - L];
   else
      sum += s[n-L];
   if (n >= N-L) 
      sum += s[L+n-N];
   else
      sum += s[n+L];
   if (n % L == 0)
      sum += s[n+L-1];
   else
      sum += s[n-1];
   if((n+1) % L == 0)
      sum += s[n-L+1];
   else
      sum += s[n+1];
   return sum;     
}

public void collectData() {
   nData++;
   Ecum += E;
   Mcum += M;
   E2cum += E*E;
   M2cum += M*M;
   Dcum += Demon;
   cf.setText(0,L);      
   if(!MC)
     T = 4.0/Math.log(1 + 4.0/(Dcum/nData));
   cf.setText(1,format(T));      
   cf.setText(2,nData);
   cf.setText(3,E);
   cf.setText(4,format(Ecum/(N*nData)));
   cf.setText(5,M);
   cf.setText(6,format(Mcum/(N*nData)));
   cf.setText(7,format((1.0/(N*T*T))*(E2cum/nData - Math.pow(Ecum/nData,2))));
   cf.setText(8,format((1.0/(N*T))*(M2cum/nData - Math.pow(Mcum/nData,2))));
}
   
public void reset() {
   Dcum=0.0;Ecum=0.0;Mcum=0.0;E2cum=0.0;M2cum=0.0;nData = 0;
   t = 0;
}


public String format(double x) {  // formats numbers to 7 characters
   String s = String.valueOf(x);
   if(s.length() > 7) 
      s = s.substring(0,6); 
   return s;
    }
}
